/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */
/*
 * Output definitions for the MStar/SigmaStar MPLL
 *
 * Copyright (C) 2020 Daniel Palmer <daniel@thingy.jp>
 */

#ifndef _DT_BINDINGS_CLOCK_MSTAR_MSC313_MPLL_H
#define _DT_BINDINGS_CLOCK_MSTAR_MSC313_MPLL_H

#define MSTAR_MSC313_MPLL_DIV2	1
#define MSTAR_MSC313_MPLL_DIV3	2
#define MSTAR_MSC313_MPLL_DIV4	3
#define MSTAR_MSC313_MPLL_DIV5	4
#define MSTAR_MSC313_MPLL_DIV6	5
#define MSTAR_MSC313_MPLL_DIV7	6
#define MSTAR_MSC313_MPLL_DIV10	7

#endif
