/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_RESET_MT6735_PERICFG_H
#define _DT_BINDINGS_RESET_MT6735_PERICFG_H

#define MT6735_PERI_RST0_UART0			0
#define MT6735_PERI_RST0_UART1			1
#define MT6735_PERI_RST0_UART2			2
#define MT6735_PERI_RST0_UART3			3
#define MT6735_PERI_RST0_UART4			4
#define MT6735_PERI_RST0_BTIF			5
#define MT6735_PERI_RST0_DISP_PWM_PERI		6
#define MT6735_PERI_RST0_PWM			7
#define MT6735_PERI_RST0_AUXADC			8
#define MT6735_PERI_RST0_DMA			9
#define MT6735_PERI_RST0_IRDA			10
#define MT6735_PERI_RST0_IRTX			11
#define MT6735_PERI_RST0_THERM			12
#define MT6735_PERI_RST0_MSDC2			13
#define MT6735_PERI_RST0_MSDC3			14
#define MT6735_PERI_RST0_MSDC0			15
#define MT6735_PERI_RST0_MSDC1			16
#define MT6735_PERI_RST0_I2C0			17
#define MT6735_PERI_RST0_I2C1			18
#define MT6735_PERI_RST0_I2C2			19
#define MT6735_PERI_RST0_I2C3			20
#define MT6735_PERI_RST0_USB			21

#define MT6735_PERI_RST1_SPI0			22

#endif
