/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
#ifndef _UAPI_LINUX_TYPELIMITS_H
#define _UAPI_LINUX_TYPELIMITS_H

#define __KERNEL_INT_MAX ((int)(~0U >> 1))
#define __KERNEL_INT_MIN (-__KERNEL_INT_MAX - 1)

#endif /* _UAPI_LINUX_TYPELIMITS_H */
