/* SPDX-License-Identifier: GPL-2.0 */

#ifndef __ASM_GENERIC_SPINLOCK_TYPES_H
#define __ASM_GENERIC_SPINLOCK_TYPES_H

#include <asm-generic/qspinlock_types.h>
#include <asm-generic/qrwlock_types.h>

#endif /* __ASM_GENERIC_SPINLOCK_TYPES_H */
