// SPDX-License-Identifier: GPL-2.0+

/*
 * Copyright (C) 2025 Advanced Micro Devices, Inc.
 */

#ifndef __SOC_ISP4_MISC_H
#define __SOC_ISP4_MISC_H

#define AMDISP_I2C_ADAP_NAME "AMDISP DesignWare I2C adapter"

#endif
